#!/Library/Frameworks/Python.framework/Versions/3.6/bin/python3

# Vincent Guillet, with the Help from Tristan Guillot (OCA), Gerald Eichstaedt and Juno's PI, Candy Hansen
# Oct. 2018

# 1) Reads the MSSS Video on Juno's approach of Jupiter
# https://www.youtube.com/watch?v=XpsQimYhNkA
#
# 2) What is done:
# - extract all jpeg frame in sub-directory full/
# - analyse only those frames corresponding to real raw maps (see below)
# - extract as a function of time the (x,y) position of Jupiter and its satellites, and the distance from Juno to Jupiter (from METADATA files)
#
# Time sampling
# -------------
# The movie corresponds to an image every 7.5 min
# Real data were sampled every 30min in a short phase at the beginning, then every 15 min.
#
# Our data starts at jpeg frame 981
# 30' sampling 
#     - first frame 981 : JNCE_2016164_00C199_V01 (12/6/2016 at 02h10) 
#     - last frame 1153 : JNCE_2016164_00C242_V01 (12/6/2016 at 23h40)
#     - Only 1 frame out of 4 (starting at 981) is from raw maps. Others (3 out of 4) are simple ponderated mean between two consecutives raws maps
#     - Total : 44 frames 
# 15' sampling 
#     - first frame 1156 : JNCE_2016165_00C243_V01 (13/6/2016 at 00h10) 
#     - last frame 2812 : JNCE_2016173_00C1071_V01 (21/6/2016 At 15h10)
#     - Only 1 frame out of 2 (starting at 1156) is from raw maps. Others (1 out of 2) are simple ponderated mean between two consecutives raws maps
#     - Total : 829 frames
#
# NB: There seems to be 4 raw frames missing in the video, so an uncertainty on the timestamps of max ~ 1h.
#
# Pixelisation of the camera
# ------------------------------------
# - JunoCam has a 58 Degree FOV and 1600 x 1200 pixels
# - Each pixel correspond to 672.7 micro radians
# - The film was made with a different pixellisation than the raw images
# 	* Size of jupiter is 18 pixels in the raw data, 32 in the film
# 	* In raw data which is the last 30min photo, the distance between satellites and Jupiter is in the raw data ~ 0.55 times that in the film
# 		FILM : frame1153
# 		RAW : JNCE_2016164_00C242_V01
#

import math
import cv2
import sys
import time
import numpy as np
from matplotlib import pyplot as plt

# Intensity threshold to remove noisy part
intensity_threshold = 1

# Flag : do we skip interruption ?
skip_interruption = 0
#skip_interruption = 1

# Original position of the satellites + Jupiter
nsat = 4
ysat = [304,324,344,353]
xsat = [811,710,738,611]

if skip_interruption == 1:
	# Positions of the satellites after interruption
	ysat2 = [439,411,344,350]
	xsat2 = [1005,794,485,751]

# For masking of Jupiter
xJup = 634
yJup = 359

# True position of Jupiter
#xtrueJup = 631
#ytrueJup = 360
xtrueJup = 630
ytrueJup = 360

# Data file for timestamps of each raw map
f_date = open("time.txt","r")
# Data file for altitude of each raw map
f_alt = open("altitude.txt","r")

# Skip the first 5 lines
for i in range(4):
	rub=f_date.readline()
	rub=f_alt.readline()

# Output files
fraw_out = open("juno_raw.dat","w+")
frescaled_out = open("juno_rescaled.dat","w+")

# Header
# Order: Callisto, Ganymede, Europe, Io
fraw_out.write("# %8s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%30s\t%30s\n"% ("id","time","xC","yC","xG","yG","xE","yE","xI","yI","xJup","yJup","DJup","distance","date","file"))
frescaled_out.write("# %8s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%30s\t%30s\n"% ("id","time","xC","yC","xG","yG","xE","yE","xI","yI","date","file"))
fraw_out.write("# %8s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%30s\t%30s\n"% ("","min","m","pixels","pixels","pixels","pixels","pixels","pixels","pixels","pixels","pixels","pixels","km","YYYY-MM-DDTHH:MM:SS.SSS"," "))
frescaled_out.write("# %8s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%30s\t%30s\n"% ("","min","10^6 km","10^6 km","10^6 km","10^6 km","10^6 km","10^6 km","10^6 km","10^6 km","YYYY-MM-DDTHH:MM:SS.SSS"," "))
fraw_out.write("# %8s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%30s\t%30s\n"% (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16))
frescaled_out.write("# %8s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%30s\t%30s\n"% (1,2,3,4,5,6,7,8,9,10,11,12))

# Extract all frames
file = 'watch-juno-approach-jupiter-and-the-galilean-moons.mp4'
vidcap = cv2.VideoCapture(file)
success,image = vidcap.read()
# Grey image
image = np.array(image[:,:,0]+image[:,:,1]+image[:,:,2])*1./3.

# Size of image
nx = image.shape[1]
ny = image.shape[0]

# Coordinate grid
xpos = np.array([range(nx) for i in range(ny)])
ypos = np.array([[i for j in range(nx)] for i in range(ny)])

# Distances^2 in pixels^2 to Jupiter
dist2 = ((ypos-yJup)**2 + (xpos-xJup)**2)

count = 0
time = -30.
success = True
 

##########################################################################
# Resolution of JunoCam
scale = 672.7e-6
# The film was made with a different pixellisation than the raw images
# Size of jupiter is 18 pixels in the raw data, 32 in the film
# In raw data which is the last 30min photo, the distance between satellites and Jupiter is in the raw data ~ 0.55 +/- 0.02 times that in the film
# FILM : frame1153
# RAW : JNCE_2016164_00C242_V01
#
scale *= 18./32.
# Convert in millions of km
scale *= 1e-6
##########################################################################

##########################################################################
# Rotation of the plane of the sky 
# To bring the x axis in the revolution plane
rotate = -5.
c=np.cos(rotate/180.*np.pi)
s=np.sin(rotate/180.*np.pi)
##########################################################################

while success:

	count += 1

	# Read image
	success,image = vidcap.read()

	#imgplot=plt.imshow(image,"gray")
	#plt.show()
	#cv2.waitKey(0)
	cv2.imwrite("./full/frame%d.jpg" % count, image)     # save frame as JPEG file

	##########################################################################
	# SAMPLING
	##########################################################################
	# Video starts at 
	# 30' sampling : [981:1153] corresponding to 44 frames from JNCE_2016164_00C199_V01 (12/6/2016 at 02h10) to JNCE_2016164_00C242_V01 (12/6/2016 at 23h40)
	# 15' sampling : [1156:2812] corresponding to 829 frames from JNCE_2016165_00C243_V01 (13/6/2016 at 00h10) to JNCE_2016173_00C1071_V01.json (21/6/2016 At 15h10)
	##########################################################################

	# Skip introduction on Galilee
	if count < 981: continue	

	# 30' sampling period : 1 frame out of 4 is from raw data
	if count < 1154: 
		if math.fmod(count,4) != 1 : continue
		time += 30.

	# Skip interpolated frame between the 30' and 15' sampling
	if count == 1154 or count == 1155: continue

	# The first frame of the 15' min period and last of the 30'min period
	if count == 1156:
		time += 30.

	# 15' sampling period : 1 frame out of 2 is from baw data
	if count > 1156: 
		if math.fmod(count,2) != 0 : continue
		time += 15.
	
	# Skip interruption if asked to
	# Do not use : after interruption, inclination angle of Juno w/r plane of revolution changes => difficult to renormalize positions
	if (skip_interruption == 1):
		if (count > 2812) & (count < 2840): 
			# Skip these files
			if math.fmod(count,2) == 0 : 
				alt=f_alt.readline()
				date=f_date.readline()
			continue
		# Init new positions after interruption: JNCE_2016176_00C1089_V01 2016-06-24T00:10:02.538
		if (count == 2840):	
			# Pb : film = interruption end at 2840
			# Pb : METADATA = interruption end at 2848
			# It looks like 4 frames have not been put in the film during the 1156-2812 period (843 frames for the film, 847 frames for METADATA)
			# Skip the 4 METADATA lost
			for i in range(4):
				alt=f_alt.readline()
				date=f_date.readline()
			xsat = xsat2
			ysat = ysat2
			# 12 days -2h after start (frame 981)
			time = 12 * 24 * 60 - 2 * 60
	
		########################## 
		# End of data, the sign Juno appears
		if (count > 3616):
			continue
	else: 
		# END OF DATA
		if (count > 2812): continue
	##########################################################################

	# Put PIXELS with intensity < 10 to zero to limit noise
	ret,image = cv2.threshold(image,intensity_threshold,255,cv2.THRESH_TOZERO)
	# Extract and convert to float
	image = np.array(image[:,:,0])*1.

	# Read METADATA : altitude and timestamp
	alt=f_alt.readline()
	date=f_date.readline()
	stralt=alt.split()
	strdate=date.split()
	altitude = float(stralt[1])
	print("Frame",count,"File=",stralt[0],"Altitude=",stralt[1],"TimeStamp=",strdate[1])

	# Initialize altitude : that of the 1st frame
	if count == 981: alt_init = altitude

	# Count the number of pixels of Jupiter to get the diameter in pixels (sqrt(8*N/pi))
	# This can be used to convert pixels into distance
	pixJup = np.where((dist2 < (20*alt_init/altitude)**2) & (image > 20))
	DJupx = np.max(xpos[pixJup])-np.min(xpos[pixJup])+1
	DJupy = np.max(ypos[pixJup])-np.min(ypos[pixJup])+1

	# Create a copy of this image
	image_save = image*1.
	# Write a cross to indicate jupiter's position for checks 
	for i in range(31): image_save[yJup+i-15][xJup] = 255
	for i in range(31): image_save[yJup][xJup+i-15] = 255
	#imgplot=plt.imshow(image_save,"gray")
	#plt.show()
	#cv2.waitKey(0)
	# Save this image in the ./work sub-directory
	cv2.imwrite("./work/frame%d.jpg" % count, image_save)     # save frame as JPEG file

	# Erase Jupiter disc to better follow satellites trajectory
	jJup = np.where((dist2 < (12.5*alt_init/altitude)**2)) #& (xpos >= xJup))
	image[jJup] = 0
	#imgplot=plt.imshow(image,"gray")
	#plt.show()
	#cv2.waitKey(0)
	# Save this image in the ./mask sub-directory
	cv2.imwrite("./mask/frame%d.jpg" % count, image)     # save frame as JPEG file

	fraw_out.write("%10d\t%10f\t"% (count,time))
	frescaled_out.write("%10d\t%10f\t"% (count,time))

	# Loop over satellites
	for k in range(nsat):

		mask = image*0.

		# Mask around satellite k
		distsat = ((ypos-ysat[k])**2 + (xpos-xsat[k])**2)
		z = np.where(distsat < 10**2)
		mask[z[0],z[1]] = 1.

		# Compute the position of the satelite using the mask
		tot = np.sum(mask*image)
		if (tot > 0):
			xsat[k] = np.vdot(xpos,mask*image)/tot
			ysat[k] = np.vdot(ypos,mask*image)/tot
			fraw_out.write("%10f\t%10f\t"% (xsat[k],ysat[k]))
			frescaled_out.write("%10f\t%10f\t"% (scale*altitude*(c*(xsat[k]-xtrueJup)+s*(ysat[k]-ytrueJup)),scale*altitude*(-s*(xsat[k]-xtrueJup)+c*(ysat[k]-ytrueJup))))
		else:
			# The position of the satellite could not be determined
			fraw_out.write("%10s\t%10s\t"% ("NaN","NaN"))
			frescaled_out.write("%10s\t%10s\t"% ("NaN","NaN"))

	fraw_out.write("%10f\t%10f\t%10f\t"% (xtrueJup,ytrueJup,DJupy))

	fraw_out.write("%10s\t%30s\t%30s\t"% (altitude,strdate[1],stralt[0]))
	frescaled_out.write("%30s\t%30s\t"% (strdate[1],stralt[0]))

	fraw_out.write("\n")
	frescaled_out.write("\n")

	
fraw_out.close() 
frescaled_out.close() 

